var additionalPageList = [];
var mainInformationPageList = [];
var hotspotList = [];
var markerList = [];
var map;
var watchId = 0;
var currentLocationMarker;
var googleMapsAPIUrl = 'http://maps.google.com/maps/api/js?sensor=true&callback=mapScriptLoaded&async=2&language=' + getCurrentLanguage();
var isRunningOnPhoneGap = checkIsRunningOnPhoneGap();
var oldAndroidPopupHeight = 0;
var goToMyLocationAfterUpdate = false;
var imageWidth = 0;

/*******************************************************************************************************
 * Event listeners
 *******************************************************************************************************/
if(isRunningOnPhoneGap) {
	document.addEventListener("deviceready", readyForLaunch, false);
} else {
	$(document).ready(function(){readyForLaunch();});
}


/**
 * Device is ready, execute the necessary init and get data functions
 */
function readyForLaunch()
{
	// Splash Screen Duration: for auto-hide behaviour call navigator.splashscreen.hide(); in the device-ready method
	// source: https://build.phonegap.com/docs/config-xml
	if(checkIsRunningOnPhoneGap()) {
		navigator.splashscreen.hide();
	}
		
	setApplicationName();	
	resizeIntroScreen();
	if(hasConnection()) {
		initApplication();	
	}
}

/**
 * Define the event listeners that will handle the user actions
 */
function defineEventListeners() 
{	
	$('a').on('click', function(e) {	

		if( hasAttributeValue($(this), 'data-type', 'informationPage')) {
			updatePaginationCurrentStatus($(this).attr('data-pagesortkey'));
			window.location.href = $(this).attr('href');					
		}
		
		if( hasAttributeValue($(this), 'data-type', 'additionalPage')) {
			window.location.href = $(this).attr('href');			
		}		
		
		if( hasAttributeValue($(this), 'data-type', 'mainMenuToggle')) {
			$(this).parent().toggleClass('main-app-selected');					
		}	
		e.preventDefault();
		return false;	
	});	
	
	// handle popup content depending on hash tag in URL
	$(window).hashchange(function(){
		storeData();
		handleHashTagForPopup();		
	});	
		
	// Resize home screen
	$(window).resize(function(){
		resizeIntroScreen();
	});	
	
	// Since the event is only triggered when the hash changes, we need to trigger
	// the event now, to handle the hash the page may have loaded with.
	$(window).hashchange();	
}

/**
 * Swipe between the pages
 */
function definePageNavigationListeners()
{
	var swipeable = $('.cb-content');	
	if(!$('#mapClickPopup').hasClass('additional-page') && $('#mapClickPopup').hasClass('information-page')) {
	
		var windowWidth = $(window).width();
		var horizontalDirection = '';
		var horizontalOppositeDirection = '';
		var minHorizontalSwipe = parseInt(windowWidth/5);

		swipeable.swipe({

			allowPageScroll:"vertical",
			swipeStatus:function(event, phase, direction, distance, duration, fingerCount){

				if(phase == 'end' && distance < minHorizontalSwipe) {

					// Reset position if not swipped enough
					swipeable.animateWithCss( { left: 0, right: 0 }, 200, "linear");

				} else if(direction=='left' || direction=='right') {

					if(direction=='left') {
						horizontalDirection = 'left';
						horizontalOppositeDirection = 'right';				
					} else {
						horizontalDirection = 'right';
						horizontalOppositeDirection = 'left';								
					}				

					// move the box following the user's swipe action
					swipeable.css(horizontalOppositeDirection, distance + 'px');
					swipeable.css(horizontalDirection, (-1 * distance) + 'px');
				}
			},
			swipe:function(event, direction, distance, duration, fingerCount){	
				if(direction=='left' || direction=='right') {
					
					var load = 'prev';
					horizontalDirection = 'right';
					horizontalOppositeDirection = 'left';				

					if(direction=='left') {
						load = 'next';
						horizontalDirection = 'left';
						horizontalOppositeDirection = 'right';				
					}

					// Move out of the window			
					var moveOutAnimation = {};
					moveOutAnimation[horizontalDirection] = (-1*windowWidth) + 'px';
					moveOutAnimation[horizontalOppositeDirection] = (windowWidth) + 'px';			
					swipeable.animateWithCss(moveOutAnimation, 300, "linear", function(){					

						// reset position outside the window on the other side
						swipeable.css(horizontalDirection, windowWidth + 'px');		
						swipeable.css(horizontalOppositeDirection, (-1*windowWidth) + 'px');					

						// change content
						$('.sub-nav a[data-pagesortkey='+$(this).parent().attr('data-'+load)+']').trigger('click');

						// move back in the window
						swipeable.animateWithCss({left:0,right:0},500);
					});
				}
			},
			threshold:minHorizontalSwipe
		});

	} else {
		swipeable.swipe("destroy");
	}	
}

function updatePaginationCurrentStatus(index) 
{	
	$('.page-nav li a').each(function(){
		if($(this).hasClass('current')) $(this).removeClass('current');
	});
	$('.page-nav li:eq('+(index-1)+') a').addClass('current');	
}


/**
 * Populate the pagination menu with HTML list items
 */
function populatePagination() 
{	
	for(var i=0; i<mainInformationPageList.length; i++) {	
		$('.page-nav').append('<li><a href="#page-'+mainInformationPageList[i].id+'" data-type="informationPage" data-pagesortkey="'+(i+1)+'"><span>'+mainInformationPageList[i].title+'</span></a></li>');
	}		
}

/*
 * Alter the url according to the anchor's href attribute, and
 * store the data-foo attribute information with the url
 * 
 */ 
function bindNavigation(anchorTagSelector)
{
	$.mobile.navigate(anchorTagSelector.attr("href"));
}

/**
 * All hotspots are on the map
 */
function mapAndHotspotsLoaded()
{		
	$('#mapCanvas').find('a').on('click', function(e) {	
		loadURL($(this).attr('href'));
		e.preventDefault();
		return false;	
	});	
	
	hideIntroScreen();	
	showTopBar();	
	defineEventListeners();		
}

/**
 * Go to the current location of the user
 */
function goToMyLocation()
{
	map.setZoom(17);
	map.panTo(currentLocationMarker.position);
}

/**
 * If the center and zoom level can be retrieved from the local storage then we use it,
 * otherwise find use the calculated center and default zoom level.
 */
function setMapCenter(calculatedCenter)
{
	if(typeof window.localStorage['mapView'] !== 'undefined') {		
		storedMapOptions = JSON.parse( window.localStorage['mapView'] );
		mapDefaultOptions['center'] = new google.maps.LatLng(storedMapOptions.center.lat,storedMapOptions.center.lng);
		mapDefaultOptions['zoom'] = storedMapOptions.zoom;			
			
	} else {
		mapDefaultOptions['center'] = new google.maps.LatLng(calculatedCenter.lat,calculatedCenter.lng);
		mapDefaultOptions['zoom'] = 12;		
	}
	map.setOptions(mapDefaultOptions);
}


/*******************************************************************************************************
 * Handle the data
 *******************************************************************************************************/

/**
 * Store the data in the local storage for later usage
 */
function storeData()
{
	if(map) {
		var mapView = {center:{lat:map.getCenter().lat(),lng:map.getCenter().lng()}, zoom:map.getZoom()};
		window.localStorage.setItem("mapView",  JSON.stringify(mapView));
	}
}

/**
 * Set the application name in different place in the app code
 */
function setApplicationName() 
{
	$('title').text(appName);
	$('.btn-app b').text(appName);	
}

/**
 * Erase the content of the popup
 */
function resetPopup()
{
	// Empty the divs 
	$('.cb-header h1').text('');		
	$('.cb-content figure').html('');	
	$('.cb-content .article-content').html('');
	
	// Delete navigation data for information pages 
	if($('#mapClickPopup').hasClass('information-page')) {
		if (typeof $('#mapClickPopup').attr('data-prev') !== 'undefined' && $('#mapClickPopup').attr('data-prev') !== false) {
			$('#mapClickPopup').removeAttr('data-prev');
		}	
		if (typeof $('#mapClickPopup').attr('data-next') !== 'undefined' && $('#mapClickPopup').attr('data-next') !== false) {
			$('#mapClickPopup').removeAttr('data-next');
		}						
		$('#mapClickPopup').removeClass('information-page');
		$('#mapClickPopup').removeClass('additional-page');
		hidePagination();		
	}
	else 
	{
		// Remove theme classes
		for(var i=1; i<5; i++) {  
			// let's assume that there are not more than 5 themes for now.
			if($('#mapClickPopup').hasClass('theme-' + i)) {
				$('#mapClickPopup').removeClass('theme-' + i); 
			}
		}
	}
}

/**
 * Add the set of data to the popup
 */
function populatePopup(data) 
{	
	oldAndroidPopupHeight = 0;
	
	
	resetPopup(); // be sure to start from scratch
	
	// Add the title, image and content
	$('.cb-header h1').text(data.title);	

	if(data.headerImage!='' && data.headerImage != null) {
		$('.cb-content > figure').append('<img src="'+apiBaseURL + 'images/content/' + data.headerImage+'" width="'+imageWidth+'">');	
		
		$('.cb-content > figure > img').on('load', function() {
			
			// Adjust the width of the image
			if(imageWidth == 0) { 
				imageWidth = $('.cb-content').width();
				$(this).attr('width', imageWidth);	
			}								
			
			if(isLessThanAndroid4()) {
				oldAndroidPopupHeight += $(this).height();

				$('.oldAndroid').find('#mapClickPopup-popup').css('height', parseInt(oldAndroidPopupHeight + 8) + 'px' );
				$('.oldAndroid').find('#mapClickPopup').css('height', parseInt(oldAndroidPopupHeight) + 'px' );				

				hideLoadingIcon();				
			}	
		});
		
	}
	$('.cb-content .article-content').append(data.content);		
	
	// Links should open in browser
	$('.cb-content .article-content').find('a').each(function(){
		$(this).attr('data-external', 'true');
		$(this).attr('data-rel', 'external');
	});
	
	// Add the theme style OR info page navigation data
	if(typeof data.themeId != 'undefined') {		
		$('#mapClickPopup').addClass('theme-' + data.themeId);
	} else {						
		if(data.isAdditional == 0) {
			$('#mapClickPopup').addClass('information-page');
			setPaginationControl(data);		
		}
		if(data.isAdditional == 1) {
			$('#mapClickPopup').addClass('information-page additional-page');
		}		
	}
	
	// Add sub pages blocks
	if(typeof(data.subPages)!=="undefined" && data.subPages.length>0) {
		for(var i=0; i<data.subPages.length; i++) {
			
			var newInfoBlock = $('#mapClickPopup-popup > .info-block').clone();
			newInfoBlock.removeClass('info-block-base');
			newInfoBlock.children('h2').append(data.subPages[i].title);
			newInfoBlock.children('.info-content').html(data.subPages[i].content);		
			newInfoBlock.addClass('info-block-'+data.subPages[i].sortKey);
			$('.cb-content .article-content').append(newInfoBlock);		
		
			// show that sub block
			$('.info-block-'+data.subPages[i].sortKey).show();			
		
			// add images 
			if(data.subPages[i].headerImage!='' && data.subPages[i].headerImage != null) {
				$('.info-block-'+data.subPages[i].sortKey).find('.info-content').prepend('<figure class="image-wrapper"><img src="'+apiBaseURL + 'images/content/' + data.subPages[i].headerImage+'"></figure>');	
				$('.info-block-'+data.subPages[i].sortKey).find('.info-content').children('figure').children('img').on('load', function() {

					// Adjust the width of the image
					$(this).attr('width', '90%');			

					if(isLessThanAndroid4()) {
						oldAndroidPopupHeight += $(this).height();

						// @todo: put this in a resize method

						$('.oldAndroid').find('#mapClickPopup-popup').css('height', parseInt(oldAndroidPopupHeight + 8) + 'px' );
						$('.oldAndroid').find('#mapClickPopup').css('height', parseInt(oldAndroidPopupHeight) + 'px' );				

						hideLoadingIcon();				
					}	
				});
			}				
		}

		// add the event listener
		$('.info-block h2').on('tap', function(){						
			$(this).parent().toggleClass('info-open');
			// scroll down a few pixels when clicking on a read more link 
			$('.cb-content').animate({scrollTop: ($('.cb-content').scrollTop() + 100) + 'px'}, 300);
		});
	}
	showPagination();		
	// Add the theme style OR info page navigation data
	if(typeof data.themeId != 'undefined' || data.isAdditional == 1) {		
		$('.page-nav li').hide();
	}	
	
	definePageNavigationListeners();
}

/**
 * Add the previous and next sort keys to the popup
 */
function setPaginationControl(data) {
	data.sortKey = parseInt(data.sortKey);
	if( data.sortKey > 1 && data.sortKey < mainInformationPageList.length ) {			
		nextKey = data.sortKey + 1;
		prevKey = data.sortKey - 1;			
	} else {
		if(data.sortKey == 1){
			nextKey = data.sortKey + 1;
			prevKey = mainInformationPageList.length;
		}
		if(data.sortKey == mainInformationPageList.length){
			nextKey = 1;
			prevKey = data.sortKey - 1;
		}
	}
	$('#mapClickPopup').attr('data-next', nextKey);
	$('#mapClickPopup').attr('data-prev', prevKey);			
}

/**
 * When the value behind the hashtag in the URL changes, 
 * then fill the popup with data, and show it.
 */
function handleHashTagForPopup()
{	
	var hash = ( location.hash.replace( /^#/, '' ) || 'blank' );		
	var hashData = hash.split('-');
	var type = hashData[0];
	var id = hashData[1];	
	
	if(type==='blank') {
		hidePopup();
	} else if(type==='page') {				
		showLoadingIcon();		
		populatePopup(getPageDataById(id));
		showPopup();	
	} else if(type==='hotspot') {
		showLoadingIcon();
		populatePopup(getPageDataByHotspotId(id));
		showPopup();
	}		
	
	$('.article-content').find('a').on('click', function(e) {	
		loadURL($(this).attr('href'));
		e.preventDefault();
		return false;	
	});
}

/**
 * Displays the hotspots on the map, and centers the map.
 * 
 * @param array data
 *	The list of hotspots
 */
function hotspotListReady(data) 
{

	for(var i=0; i<data.length; i++) {	
		hotspotList.push(data[i]);
	}

	for(var i=0; i<hotspotList.length; i++) {
		var marker = new google.maps.Marker({
			icon: new google.maps.MarkerImage(
				"img/icon-map-theme"+hotspotList[i].themeId+".png", 
				new google.maps.Size(24, 24), 
				null, 
				new google.maps.Point(12,12),
				new google.maps.Size(24, 24)
				),
			position: new google.maps.LatLng(hotspotList[i].lat, hotspotList[i].lng),
			title: hotspotList[i].title,
			url: '#hotspot-' + hotspotList[i].id,
			clickable: true
		});		
		marker.setMap(map);
		markerList.push(marker);
		
		google.maps.event.addListener(marker, 'click', function() {
			window.location.href = this.url;
		});				
	} 	
	
	var calculatedCenter = {
		'lat':(startingBounds['NE'][0] + startingBounds['SW'][0])/2, 
		'lng':(startingBounds['NE'][1] + startingBounds['SW'][1])/2
	};
	setMapCenter(calculatedCenter);
	
	mapAndHotspotsLoaded();	
}

/**
 * Adds the menu elements to the page 
 * 
 * @param array data
 *	The list of hotspots
 */
function additionalPageListReady(data) 
{	
	for(var i=0; i<data.length; i++) {	
		additionalPageList.push(data[i]);
	}
}

/**
 * Adds the menu elements to the page 
 * 
 * @param array data
 *  The list of pages
 */
function mainMenuPageListReady(data) 
{	
	for(var i=0; i<data.length; i++) {	
		mainInformationPageList.push(data[i]);
	}
	populateMenu();
	populatePagination();
}

/**
 * Populate the main menu with HTML list items
 */
function populateMenu() 
{	
	for(var i=0; i<mainInformationPageList.length; i++) {	
		$('.sub-nav').append('<li><a href="#page-'+mainInformationPageList[i].id+'" data-type="informationPage" data-pagesortkey="'+(i+1)+'"><span>'+mainInformationPageList[i].title+'</span></a></li>');
	}		
}

/**
 * Displays a crosshair marker on the map
 * 
 * @param object position
 *	The location to display the marker
 */
function onGeolocationForMapSuccess(position) 
{
	showGeoLocation();

    var markerPoint = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
    currentLocationMarker.setPosition(markerPoint);
	
	if(goToMyLocationAfterUpdate == true) {
		goToMyLocation();
	}	
}

/**
 * Geolocation error : hide control + icon
 */
function geolocationErrorHandler(error) 
{
	// hide the geolocation icon
	hideGeoLocation();
}

/*******************************************************************************************************
 * Load / Retrieve data
 *******************************************************************************************************/

/**
 * Load all the data needed for the application
 */
function initApplication()
{	
	apiRequest("mainMenuPageList", {}, [mainMenuPageListReady]);
	apiRequest("additionalPageList", {}, [additionalPageListReady]);	
	loadMap();		
}

/*
 * Check if Google Maps is loaded, 
 * if not, load it.
 */
function loadMap()
{
	if(typeof google === 'object' && typeof google.maps === 'object'){
		mapScriptLoaded();
	} else {
		$.getScript(googleMapsAPIUrl);
	}		
}
/**
 * Apply configuration to the map object
 */
function mapScriptLoaded() 
{	
	mapDefaultOptions = {
		panControl: false,		
		scaleControl: false,
		zoomControl: true,
		zoomControlOptions: {
			style: google.maps.ZoomControlStyle.SMALL,
			position: google.maps.ControlPosition.LEFT_TOP
		},	
		streetViewControlOptions: {			
			position: google.maps.ControlPosition.LEFT_TOP
		},		
		mapTypeId: google.maps.MapTypeId.ROADMAP,
		styles: mapStyles
	}
	map = new google.maps.Map(document.getElementById("mapCanvas"),mapDefaultOptions);
	
	currentLocationMarker = new google.maps.Marker({
		map: map,
		icon: new google.maps.MarkerImage("img/icon-marker2_at_2x.png", 
			new google.maps.Size(48, 48), 
			null, 
			new google.maps.Point(24,48),
			new google.maps.Size(48, 48)
			),		
		optimized: false,
		clickable: false
	});	
	
	// Current location of the app user.
	goToMyLocationAfterUpdate = false;
	getLocationUpdate();

	// Event listener: map style
	google.maps.event.addListener(map, 'maptypeid_changed', onMapTypeIdChange);

	// Load all the hotspots
	apiRequest("hotspotList", {
		'theme':defaultThemeId
	}, [hotspotListReady]) ; 		
}

/**
 * Get the latest position of the user
 */
function getLocationUpdate() 
{	
	if(navigator.geolocation) {
		// timeout at 10000 milliseconds (10 seconds)
		var options = {
			enableHighAccuracy: false,
			timeout: 10000,
			maximumAge: 0
		};
		watchId = navigator.geolocation.getCurrentPosition(onGeolocationForMapSuccess,geolocationErrorHandler,options);
	}else{
		hideGeoLocation();
	}
}


/**
 * Get the Page data for a given Page, 
 * from nthe main list array or the information page list
 * 
 * @param id of the Page
 * @return object Page data
 */
function getPageDataById(id)
{
	for(var i=0; i<mainInformationPageList.length; i++) {	
		if(mainInformationPageList[i].id == id) {
			return mainInformationPageList[i];
		}
	}
	for(var i=0; i<additionalPageList.length; i++) {	
		if(additionalPageList[i].id == id) {
			return additionalPageList[i];
		}
	}	
	return false;
}

/**
 * Get the Page data for a given Hotspot
 * 
 * @param id of the Hotspot
 * @return object Page data
 */
function getPageDataByHotspotId(id)
{
	for(var i=0; i<hotspotList.length; i++) {	
		if(hotspotList[i].id == id) {
			return hotspotList[i];
		}
	}
	return false;
}


/**
 * Resize into screen
 */
function resizeIntroScreen()
{
	if($('div.home').length) {
		$('.home').css('min-height', $('.ui-page-active').css('min-height'));
	}	
}

/**
 * Resize elements within the popup
 */
function resizePopupElements() 
{	
	// repositionate the content area, depending on the header size
	$('.cb-content').css('top', $('.cb-header').outerHeight());
		
	// Force youtube iframe to be full width and 16:9 format 
	$('.cb-content').find('iframe').each(function(){		
		$(this).attr('width', parseInt($('.article-content').width() - 20));
		$(this).attr('height', parseInt(($('.article-content').width() - 20) * 9 / 16) );
	});
}

/*******************************************************************************************************
 * Show / Hide UI elements
 *******************************************************************************************************/

/**
 * Show application top nav bar
 */
function showTopBar() 
{
	$('.topbar').show();
	hideMainMenu();			
}

/**
 * Show pagination (for information pages)
 */
function showPagination() 
{
	$('.page-nav li').show();
	$('.cb-footer').show();
}

/**
 * Show the popup container
 */
function showPopup()
{
	hideMainMenu();
	
	$('#mapClickPopup-popup').show(); 
	
	var overlayHeight = parseInt($('.ui-page-active').css('min-height'), 10) - parseInt($('.googlemap-overlay').css('top'),10);
	$('.googlemap-overlay').animate({opacity:0.75},0).show().css('min-height', overlayHeight+'px');
	
	
	
	$('#mapClickPopup-popup > .info-block-base').hide();	
	resizePopupElements();					
			
	
	
	if(isLessThanAndroid4()) {	
		$('body').addClass('oldAndroid');
		$('.oldAndroid').find('.cb-content').css('top', 0);	
		
		// scroll back to top on click on pagination links
		$('.oldAndroid').find('.page-nav li a').on('click', function(){
			$(window).scrollTop(0);
		});

		// Open all subpages
		$('.oldAndroid').find('.info-block').each(function(){
			$(this).addClass('info-open');
		});
			
		oldAndroidPopupHeight += $(".cb-header").outerHeight();
		oldAndroidPopupHeight += $(".article-content").outerHeight();	
		oldAndroidPopupHeight += $(".cb-footer").outerHeight();
		
		if($("iframe").length > 0) {
			$("iframe").each(function(){
				oldAndroidPopupHeight += $(this).height();	
			});					
		}			
		
		$('.oldAndroid').find('#mapClickPopup-popup').css('height', parseInt(oldAndroidPopupHeight + 8) + 'px' );
		$('.oldAndroid').find('#mapClickPopup').css('height', parseInt(oldAndroidPopupHeight) + 'px' );							
	
	}	
	else {
		// Scroll back to the top	
		$(".cb-content").scrollTop(0);		
		
	}
	hideLoadingIcon();
}

/**
 * Show the loading icon
 */
function showLoadingIcon()
{
	$('.ui-loader').delay(300).show();
}

/**
 * show the geo location control + marker
 */
function showGeoLocation() 
{
	// Location button
	$('.my-location').show();	
	
	// Location marker
	currentLocationMarker.setMap(map);
		
	// On click on the go to my location button, zoom in the area around the user's current locaiton
	$('.my-location').on('click', function(){			
		goToMyLocationAfterUpdate = true;
		getLocationUpdate();		
	});
}

/**
 * Hide the intro screen
 *  - add a little delay in order to hide the loading icon
 */
function hideIntroScreen()
{
	setTimeout(function(){		
									
		$('div.home').animateWithCss({opacity:0},500,"linear", function(){
			hideLoadingIcon();
			$(this).remove();
		});
	}, 3000);	
}

/**
 * Hide pagination (for information pages)
 */
function hidePagination() 
{
	$('.cb-footer').hide();
}

/**
 * Hide the loading icon
 */
function hideLoadingIcon()
{
	$('.ui-loader').hide();
}

/**
 * Hide the popup container, and reset its content.
 */
function hidePopup()
{	
	goToMyLocationAfterUpdate = false;
	getLocationUpdate();
	
	//showMapControls();
	$('#mapClickPopup-popup').hide();
	$('.googlemap-overlay').animate({opacity:0},0).hide();
}

/**
 * Hide the main menu
 */
function hideMainMenu()
{
	$('.main-app').removeClass('main-app-selected');
}

/**
 * hide the geo location control + marker
 */
function hideGeoLocation() 
{
	// Location button
	$('.my-location').hide();	
	
	// Location marker
	currentLocationMarker.setMap(null);
}