/**
 * Check whether the code is being run within PhoneGap (as a native app)
 */
function checkIsRunningOnPhoneGap() 
{
	if ((document.URL.indexOf('http://') === -1) && (document.URL.indexOf('https://') === -1)) {
		if (navigator.userAgent.match(/(iPhone|iPod|iPad|Android|BlackBerry)/)) {
			return true;
		} else {	
			return false;
		}
	} else {
		return false;
	}
}

function loadURL(url)
{		
	if(!checkIsRunningOnPhoneGap()) {
		window.open(url, '_blank');
	} else {
		window.open(url, '_system');
	}		
}

/**
 * Get the current language of the application.
 * 
 * @return string 2 letters language code
 *
 */
function getCurrentLanguage()
{
	return defaultLanguage;
}

/**
 *
 */
function hasAttributeValue(object, attributeName, attributeValue) 
{
	if( hasAttribute(object, attributeName) ) {
		if(object.attr(attributeName) == attributeValue) {
			return true;
		}				
	}
	return false;
}

function hasAttribute(object, attributeName) 
{
	if (typeof object.attr(attributeName) != 'undefined' && object.attr(attributeName) != false) {		
		return true;
	}
	return false;
}


/**
 * Get the message from the applicationMessages object in the current language
 * 
 * @param string messageCode
 * 
 * @return string message
 */
function getMessage(messageCode)
{	
	if(typeof applicationMessages[messageCode] == 'undefined' || typeof applicationMessages[messageCode][getCurrentLanguage()] == 'undefined') {
		return '';
	}
	return applicationMessages[messageCode][getCurrentLanguage()];
}

/**
 * If the user chooses another map style, then change the map options object.
 */
function onMapTypeIdChange() {
	if(map.getMapTypeId() == google.maps.MapTypeId.ROADMAP) {
		map.setOptions({ styles: mapStyles });
	} else {
		map.setOptions({ styles: [] });
	}
}

/**
 * Makes a request to the API
 * 
 * @param string method
 *	The API method
 * @param array parameters
 *	The parameters to give to the api
 * @param function array success
 *	The functions to call upon success
 * @param function array error
 *	The functions to call upon error
 * @param bool showLoadingIcon
 *	Show the loading icon during the request
 * @param bool handleErrors
 *	Show an error message if the request fails
 * @return object
 *	The jQuery XMLHttpRequest used for performing the request
 */
function apiRequest(method, parameters, success, error, showLoadingIcon, handleErrors) 
{
	
	if (showLoadingIcon) {
		$.mobile.loading('show');
	}

	var jqxhr = $.getJSON( apiBaseURL + getCurrentLanguage() + '/api/' + method );
	jqxhr.success(success);
	jqxhr.error(error);
	if (handleErrors) {
		jqxhr.error(function() {
			alert(messages.noDataAvailable.en);
		} );
	}
	if (showLoadingIcon) {
		jqxhr.complete(function() {
			$.mobile.loading('hide');
		});
	}
	return jqxhr;
}

/**
 * If there is no connection, then display a warning message
 * 
 * @return boolean
 */
function hasConnection()
{
	if( (!isRunningOnPhoneGap && navigator.onLine == false) 
		|| (isRunningOnPhoneGap && ( navigator.connection.type == Connection.NONE || navigator.connection.type == Connection.UNKNOWN)) 
	){
		alertMessage(getMessage('noConnection'));
		return false;
	} else {
		return true;
	}	
}

/**
 * Alert a message 
 */
function alertMessage(message) 
{
	if(isRunningOnPhoneGap) {
		navigator.notification.alert(message, function(){});
	} else {
		alert(message);
	}
}

/**
 * Is android version less than 4 ?
 */
function isLessThanAndroid4()
{
	if(isRunningOnPhoneGap) {	
		var deviceOSVersion = device.version;  //fetch the device OS version	
		if( parseInt( deviceOSVersion, 10 ) < 4 )
		{
			return true;
		}	
	}
	return false;
}
